#include <SFML/Graphics.hpp>
#include <string>
#include <sstream>

#include "player.h"
#include "display.h"
#include "level.h"
#include "explore.h"
#include "font.h"
#include "encounter.h"
#include "misc.h"

using namespace std;
using namespace sf;

extern Player plyr;
extern string roomMessages[255];
extern string descriptions[255];
extern weapon Weapons[56];

// Main window
sf::RenderWindow App;

//sf::Font myfont;

bool mainMenuQuit = false;


int depth = 13; // should be 13
int columns = 25; // should be an odd number 25
int frontwall = 0;
int leftwall = 0;
int rightwall = 0;
int specialwall = 0;
int zone = 0;

struct zoneRecord
{
 int wall;
 int door;
 int arch;
 int floor;     
 //int ceiling;
 //int background; // if background != 0 then single background used for all 4 directions
 //int backNorth;
 //int backSouth;
 //int backWest;
 //int backEast;
 //bool hasCeiling;
 //bool hasLight;
 //bool hasAntiMagic;
};

zoneRecord zones[19]  = {
{9,10,11,7}, // brown wall 1.2 default 0 9,10,11,7
{4,5,6,8}, // sewer1
{4,5,6,7}, // well lit area 2
{4,5,6,7}, // enhanced main 3
{12,13,14,15}, // goblin lair 4
{4,5,6,7}, // 5
{4,5,6,7}, // 6
{4,5,6,7}, //7
{4,5,6,7}, //8
{4,5,6,7},  // 9
{4,5,6,7}, // 10
{16,17,2,18}, // mausoleum 11
{12,13,2,15}, // goblin lair 12
{12,13,2,19}, // troll lair 13
{4,5,6,7}, // default 1.1 & 1.4 14 (4 5 6 7)
{20,20,20,7}, // dark wizards
{21,22,23,24}, // light wizards
{25,26,26,7},
{25,26,26,34}, // City enclosed 1?
};

// Storage for textures
const int numberOfTextures = 35;
GLuint texture[numberOfTextures];

string textureNames[numberOfTextures] = {
"wall1",
"door1",
"arch1",
"floor",
"owall",
"odoor",
"oarch",
"ofloor",
"osewerfloor1",
"obrownwall",
"obrowndoor",
"obrownarch",
"oGoblinwall",
"oGoblindoor",
"oarch",
"oGoblinFloor",
"oMausoleumWall",
"oMausoleumDoor",
"oMausoleumFloor",
"oTrollFloor",
"blackWall",
"wallGrey",
"doorGrey",
"archGrey",
"floorLightGrey",
"citywall1",
"citydoor1",
"citydoorShop",
"citydoorInn",
"citydoorTavern",
"citydoorSmithy",
"citydoorBank",
"citydoorGuild",
"citydoorHealer",
"oCityFloor1"
};

// door images - shop-27, smithy-30

sf::Image BannerImageCity, BannerImageDungeon;
sf::Image bg0,bg1,bg2,bg3,bg11,bg12,bg13,bg14,bg15,bg16,bgCity1,bgCity2,bgCity3,bgCity4;
sf::Image compassN,compassS,compassW,compassE;
sf::Image imgDamon,imgCitySmithy,imgCityTavern,imgCityShop,imgCityHealer,imgCityBank,imgCityInn,imgCityGuild, imgStairwayDown,imgStairwayUp;
sf::Sprite Background;
sf::Sprite Banner;
sf::Sprite shopImage;
sf::Sprite compass;
string olddrawText; // text string used for setting and passing strings to the print routine
       
void drawCompass()
{
	if (plyr.compasses > 0)
	{
		compass.SetPosition(16, 155);
		if (plyr.facing == 1) { compass.SetImage(compassW); }
		if (plyr.facing == 2) { compass.SetImage(compassN); }
		if (plyr.facing == 3) { compass.SetImage(compassE); }
		if (plyr.facing == 4) { compass.SetImage(compassS); }

		App.Draw(compass);
	}
}


void displayMenu()
{
	string key;
	bool gameNotStarted = true;
	while (gameNotStarted)
	{
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
		drawText(7,2,"The Alternate Reality Project");
		drawText(4,6," ) Create a new City character");
		drawText(4,8," ) Create a new Dungeon character");
		drawText(4,10," ) Quit");

		SetFontColour(40, 96, 244, 255);
		drawText (4,6,"1");
		drawText (4,8,"2");
		drawText (4,10,"Q");
		SetFontColour(255, 255, 255, 255);

		App.Display();
			
		key = getSingleKey();
		if ( key=="1" ) 
		{ 
			plyr.scenario=0;
			plyr.map=0;
			plyr.x = 36; //36
			plyr.y = 37; //37
			plyr.facing = 2;

			loadMapData(0);
			loadDescriptions(0);
			loadZoneData(0);
			loadMessages(0);
			loadCityMonsters();
			loadCityWeapons();
			gameNotStarted=false;
		}
		
		if ( key=="2" ) 
		{ 
			plyr.scenario=1;
			plyr.map=1;
			plyr.x = 50; //50
			plyr.y = 4; // 4
			plyr.facing = 1;

			loadMapData(1);
			loadDescriptions(1);
			loadZoneData(1);
			loadMessages(1);
			loadCityMonsters(); //temp
			loadCityWeapons(); //temp
			gameNotStarted=false; 
		}

		if ( key=="Q" )
		{
			mainMenuQuit = true;
			gameNotStarted=false;
		}
		
		if ( key=="QUIT" )
		{
			mainMenuQuit = true;
			gameNotStarted=false;
		}
	

	}
}


void mainLoop()
{
	// Create main window
	App.Create(sf::VideoMode(672, 512), "The Alternate Reality Project", sf::Style::Close); // was 640x480
	App.PreserveOpenGLStates(true);

	displayMenu();

 // Create sprites for the backgrounds

	bgCity1.LoadFromFile("data/images/cityBack1.png");
	bgCity2.LoadFromFile("data/images/cityBack2.png");
	bgCity3.LoadFromFile("data/images/cityBack3.png");
	bgCity4.LoadFromFile("data/images/cityBack4.png");
	bg0.LoadFromFile("data/images/oorangesky1.png"); // oorangesky1
	bg1.LoadFromFile("data/images/orangesky.png");
	bg2.LoadFromFile("data/images/osewersky1.png");
	bg3.LoadFromFile("data/images/osewersky1.png");
	bg11.LoadFromFile("data/images/oMausoleumSky.png");
	bg12.LoadFromFile("data/images/oGoblinSky.png");
	bg13.LoadFromFile("data/images/oTrollSky.png");
	bg14.LoadFromFile("data/images/ogreysky1.png");
	bg15.LoadFromFile("data/images/skyLightGrey.png");
	bg16.LoadFromFile("data/images/oCitySky1.png");

	imgDamon.LoadFromFile("data/images/oDamon.png");
	imgDamon.SetSmooth(false);
	imgCitySmithy.LoadFromFile("data/images/imgCitySmithy.png");
	imgCitySmithy.SetSmooth(false);
	imgCityTavern.LoadFromFile("data/images/imgCityTavern.png");
	imgCityTavern.SetSmooth(false);
	imgCityInn.LoadFromFile("data/images/imgCityInn.png");
	imgCityInn.SetSmooth(false);
	imgCityShop.LoadFromFile("data/images/imgCityShop.png");
	imgCityShop.SetSmooth(false);
	imgCityBank.LoadFromFile("data/images/imgCityBank.png");
	imgCityBank.SetSmooth(false);
	imgCityGuild.LoadFromFile("data/images/imgCityGuild.png");
	imgCityGuild.SetSmooth(false);
	imgCityHealer.LoadFromFile("data/images/imgCityHealer.png");
	imgCityHealer.SetSmooth(false);
	imgStairwayUp.LoadFromFile("data/images/stairwayUp.png");
	imgStairwayUp.SetSmooth(false);
	imgStairwayDown.LoadFromFile("data/images/stairwayDown.png");
	imgStairwayDown.SetSmooth(false);


	compassN.LoadFromFile("data/images/cityCompassN.png");
	compassS.LoadFromFile("data/images/cityCompassS.png");
	compassW.LoadFromFile("data/images/cityCompassW.png");
	compassE.LoadFromFile("data/images/cityCompassE.png");

	// Create a sprite for the stat banner
    //sf::Image BannerImageCity, BannerImageDungeon;
	BannerImageDungeon.LoadFromFile("data/images/dungeonBanner.png"); 
	BannerImageCity.LoadFromFile("data/images/cityBanner.png");

	if (plyr.scenario==0) { Banner.SetImage(BannerImageCity); }
	else { Banner.SetImage(BannerImageDungeon); }
    
	

    // Set color and depth clear value
    glClearDepth(1.f);
    glClearColor(0.f, 0.f, 0.f, 0.f);

    // Enable Z-buffer read and write
    glEnable(GL_DEPTH_TEST);
    glDepthMask(GL_TRUE);

    // Setup a perspective projection
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    gluPerspective(90.f, 1.f, 1.f, 500.f);


	initTextures();

    // Enable Z-buffer read and write
    //glEnable(GL_DEPTH_TEST);
    //glDepthMask(GL_TRUE);
    //glClearDepth(1.f);

	// Set The Viewport To The Top Left 1/4
	int width = 360;
	int height = 190;
	glViewport (156, 200, width, height); // measured from 0,0 bottom left of screen was 140,154
	glMatrixMode (GL_PROJECTION);				// Select The Projection Matrix
	glLoadIdentity ();							// Reset The Projection Matrix
	//gluPerspective(45.f, 1.f, 0.1f, 500.f); // was 45 degrees
	gluPerspective(45.0f,(GLfloat)width/(GLfloat)height,0.1f,100.0f);
	glTranslatef(0.0f,0.0f,-1.0f);                         // -2.4f  - move x units into the screen. was 1.0

    // Bind our texture
   
	glEnable(GL_TEXTURE_2D);
    glBindTexture(GL_TEXTURE_2D, texture[0]); // temp line
    //glColor4f(1.f, 1.f, 1.f, 1.f);


	// Clear depth buffer
	//glClear(GL_DEPTH_BUFFER_BIT);

    // Create a clock for measuring the time elapsed
    //sf::Clock Clock;

	if (!mainMenuQuit) { explore(); }
	
    // Don't forget to destroy our texture
//glDeleteTextures(1, &Texture);
   
}


void drawCellWalls(int c, int d, float xm, float zm, int frontwall, int leftwall, int rightwall) 
{
    int texture_no;
    int wall_type;

    // Y never changes in the AR world only X and Z
  
    // Draw floor
             
    //if (plyr.zone == 0) { texture_no = 3; } else {texture_no = 3;} // was 7
    if (plyr.zone != 99)
	{
		texture_no = zones[plyr.zoneSet].floor;
		glBindTexture(GL_TEXTURE_2D, texture[texture_no]);  
		glBegin(GL_QUADS);       
		glTexCoord2f(0.0f, 0.0f); glVertex3f(-25.0f+xm, -0.5f,  -25.0f+zm);	// Bottom Left Of The Texture and Quad
		glTexCoord2f(1.0f, 0.0f); glVertex3f(-23.0f+xm, -0.5f,  -25.0f+zm);	// Bottom Right Of The Texture and Quad
		glTexCoord2f(1.0f, 1.0f); glVertex3f(-23.0f+xm,  -0.5f,  -23.0f+zm);	// Top Right Of The Texture and Quad
		glTexCoord2f(0.0f, 1.0f); glVertex3f(-25.0f+xm,  -0.5f,  -23.0f+zm);	// Top Left Of The Texture and Quad
		glEnd(); 
	}

    int midcol = ((columns-1)/2);


	// left wall

    if ((!(leftwall<1)) && (c <= midcol))  
    {
        wall_type = leftwall;     
        if (( wall_type == 1 ))
        {                     
			glEnable(GL_BLEND);									// Enable Blending
			glDisable(GL_DEPTH_TEST);							// Disable Depth Testing
			glBlendFunc( GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA );
		}
			texture_no = getTextureIndex(wall_type);
			glBindTexture(GL_TEXTURE_2D, texture[texture_no]);
			glBegin(GL_QUADS);		                // begin drawing walls
			glTexCoord2f(0.0f, 1.0f); glVertex3f(-25.0f+xm, -0.5f, -23.0f+zm);	// Bottom Left Of The Texture and Quad
			glTexCoord2f(1.0f, 1.0f); glVertex3f(-25.0f+xm, -0.5f, -25.0f+zm);	// Bottom Right Of The Texture and Quad
			glTexCoord2f(1.0f, 0.0f); glVertex3f(-25.0f+xm,  0.5f, -25.0f+zm);	// Top Right Of The Texture and Quad
			glTexCoord2f(0.0f, 0.0f); glVertex3f(-25.0f+xm,  0.5f, -23.0f+zm);	// Top Left Of The Texture and Quad
			glEnd();
		 if (( wall_type == 1 )) // was 1
		 {   
			glEnable(GL_DEPTH_TEST);							// Enable Depth Testing
			glDisable(GL_BLEND);                   
		 }                                
}
	
    if ((!(rightwall<1)) && (c >= midcol))
    {
        wall_type = rightwall;   
        if (wall_type==1)
		{
			glEnable(GL_BLEND);									// Enable Blending
			glDisable(GL_DEPTH_TEST);							// Disable Depth Testing
			glBlendFunc( GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA );
		}
	   texture_no = getTextureIndex(wall_type);
		glBindTexture(GL_TEXTURE_2D, texture[texture_no]);
		glBegin(GL_QUADS);		                // begin drawing walls
		glTexCoord2f(0.0f, 1.0f); glVertex3f(-23.0f+xm, -0.5f, -25.0f+zm);	// Bottom Left Of The Texture and Quad
		glTexCoord2f(1.0f, 1.0f); glVertex3f(-23.0f+xm, -0.5f, -23.0f+zm);	// Bottom Right Of The Texture and Quad
		glTexCoord2f(1.0f, 0.0f); glVertex3f(-23.0f+xm,  0.5f, -23.0f+zm);	// Top Right Of The Texture and Quad
		glTexCoord2f(0.0f, 0.0f); glVertex3f(-23.0f+xm,  0.5f, -25.0f+zm);	// Top Left Of The Texture and Quad
		glEnd();
		if (wall_type==1)
		{
			glEnable(GL_DEPTH_TEST);							// Enable Depth Testing
			glDisable(GL_BLEND);                   
		}            

 
    }


	if (!(frontwall<1)) // Ignore wall type o (clear) and 1 (arch)
	{              
		wall_type = frontwall; 
		if (wall_type==1)
		{
			glEnable(GL_BLEND);	// Enable Blending
			glDisable(GL_DEPTH_TEST);							// Disable Depth Testing
			glBlendFunc( GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA );
		}
		texture_no = 0;
		if (wall_type==3) { texture_no = checkCityDoors(); }
		if (texture_no==0) { texture_no = getTextureIndex(wall_type); }
		glBindTexture(GL_TEXTURE_2D, texture[texture_no]);
		glBegin(GL_QUADS); // begin drawing walls
		glTexCoord2f(0.0f, 1.0f); glVertex3f(-25.0f+xm, -0.5f,  -25.0f+zm);	// Bottom Left Of The Texture and Quad
		glTexCoord2f(1.0f, 1.0f); glVertex3f(-23.0f+xm, -0.5f,  -25.0f+zm);	// Bottom Right Of The Texture and Quad
		glTexCoord2f(1.0f, 0.0f); glVertex3f(-23.0f+xm,  0.5f,  -25.0f+zm);	// Top Right Of The Texture and Quad
		glTexCoord2f(0.0f, 0.0f); glVertex3f(-25.0f+xm,  0.5f,  -25.0f+zm);	// Top Left Of The Texture and Quad
		glEnd();
		if (wall_type==1)
		{
			glEnable(GL_DEPTH_TEST);	// Enable Depth Testing
			glDisable(GL_BLEND);      
		}
    }


}

int checkCityDoors() // currently only forward!!!
{
	return 0;
}



void drawInfoPanels()
{
    if (plyr.infoPanel == 1)
    { 
   
        if ( (plyr.special>=0xc0) && (plyr.special<=0xff) && (plyr.scenario==1) ) // 224 - 255 was d2
        {  
            string str; 
            int ind = (plyr.special-0xc0);
     	 	str = roomMessages[ind];
            cText(str);
     	 	
        }
        else
        {
			bText(2, 1, "Food Packets    Torches   Water Flasks");
            bText(7, 2,plyr.food);
            bText (20, 2, plyr.torches);
            bText (33, 2, plyr.water);
            if ( plyr.scenario == 0)
            {
                  bText(11, 4, "You are in the city");
                  bText(12, 5, "of Xebec's Demise");   
            }
            else
            {
                bText(13, 4, "You are on level 1");
                bText(15, 5, "of the Dungeon");
            }
        }
    }
    
    
       if (plyr.infoPanel == 2)
    {
    
//        int weapon = objectBuffer[plyr.priWeapon].index; 
        //drawTop();
        bText(1, 1, "Gold Coins   Silver Coins   Copper Coins");
        bText(3, 2,plyr.gold);
		bText(17, 2,plyr.silver);
		bText(32, 2,plyr.copper);
		
		
		//drawText(1, 1, "  %d             %d             %d",plyr.gold, plyr.silver, plyr.copper);
      
        bText(1, 4,"            Other Possessions");

		bText(8, 6, "Gems:");
       bText(6, 7, "Jewels:");
       bText(4, 8,"Crystals:"); 
       bText(29, 6,"Keys:"); 
       bText(24, 7,"Compasses:");
       bText(23, 8,"Timepieces:");



       // drawText (7, 5, "Gems: %d", plyr.gems);
       // drawText (5, 6, "Jewels: %d", plyr.jewels);
       // drawText (3, 7,"Crystals: %d",plyr.crystals); 
       // drawText (28, 5,"Keys: %d",plyr.keys); 
       // drawText (23, 6,"Compasses: %d",plyr.compasses);
       //  drawText (22, 7,"Timepieces: %d",plyr.timepieces);          
    }
    
    if (plyr.infoPanel == 3)
    {
    
//        int weapon = objectBuffer[plyr.priWeapon].index; 
        //drawTop();
        bText(18, 1, "Weapons");
		string	str = "Primary: "+Weapons[plyr.priWeapon].name;
		bText (1, 2, str);
		str = "Secondary: "+Weapons[plyr.secWeapon].name;
		bText (1, 3, str);
	/*	if (plyr.priWeapon == 255) { drawText (1, 1, "Primary: Bare hand"); }
		else { drawText (1, 1, "Primary: %s", weapons[weapon].name.c_str()); }
        weapon = objectBuffer[plyr.secWeapon].index;
		if (plyr.secWeapon == 255) { drawText (1, 2, "Secondary: Bare hand"); }
		else { drawText (1, 2, "Secondary: %s", weapons[weapon].name.c_str()); } */
        bText (19, 4, "Armour");
        bText (1, 5, "Head: None");
        bText (1, 6,"Body: None"); 
        bText (1, 7,"Arms: None"); 
        bText (1, 8,"Legs: None");            
    }
}


void drawShopImage(string imagename)
{	
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	if (imagename=="imgDamon") shopImage.SetImage(imgDamon); // imgDamon 
	if (imagename=="imgCitySmithy") shopImage.SetImage(imgCitySmithy);
	if (imagename=="imgCityTavern") shopImage.SetImage(imgCityTavern);
	if (imagename=="imgCityInn") shopImage.SetImage(imgCityInn);
	if (imagename=="imgCityBank") shopImage.SetImage(imgCityBank);
	if (imagename=="imgCityGuild") shopImage.SetImage(imgCityGuild);
	if (imagename=="imgCityShop") shopImage.SetImage(imgCityShop);
	if (imagename=="imgCityHealer") shopImage.SetImage(imgCityHealer);
	if (imagename=="imgStairwayUp") shopImage.SetImage(imgStairwayUp);
	if (imagename=="imgStairwayDown") shopImage.SetImage(imgStairwayDown);
	
	shopImage.SetPosition(16, 144); //16,140
	App.Draw(shopImage);
	
}


void drawStatsPanel()
{
	//Draw stat banner
	if (plyr.scenario==0) { Banner.SetImage(BannerImageCity); }
	else { Banner.SetImage(BannerImageDungeon); }
	Banner.SetPosition(16, 16);
	App.Draw(Banner);
	
	if (plyr.scenario==1) { SetFontColour(225, 162, 168, 255); }
	else { SetFontColour(184, 128, 0, 255); }

	drawText(2, 1, plyr.name);
	drawText(32,1,"Level:");
	drawText(38,1,plyr.level);

	if (plyr.scenario==1) { SetFontColour(242, 179, 244, 255); }
	else { SetFontColour(138, 65, 152, 255); }

	drawText(2,2,"Stats:  STA  CHR  STR  INT  WIS  SKL");

	if (plyr.scenario==1) { SetFontColour(202, 189, 255, 255); }
	else { SetFontColour(113, 65, 218, 255); }

	drawText(10,3,plyr.sta);
	drawText(15,3,plyr.chr);
	drawText(20,3,plyr.str);
	drawText(25,3,plyr.inte);
	drawText(30,3,plyr.wis);
	drawText(35,3,plyr.skl);

	if (plyr.scenario==1) { SetFontColour(157, 213, 255, 255); }
	else { SetFontColour(40, 96, 244, 255); }

	drawText(2,4,"Experience:");
	drawText(13,4,plyr.xp);
	if (plyr.hp < 0)
		{   
		   drawText(24,4,"Hit Points: !!!");   
		}
		else
		{
			drawText(25,4,"Hit Points:");
			drawText(36,4,plyr.hp);
		}

	std::string str;
	str = "You are "+ descriptions[plyr.location];

	if (plyr.scenario==1) { SetFontColour(168, 241, 154, 255); }
	else { SetFontColour(111, 159, 6, 255); }

	if (plyr.alive) { drawText(2,5,str); }
	else { drawText(14,5,"Where are you?"); }

    SetFontColour(255, 255, 255, 255); // set text colour to white for all other text
	
	// Diag Text
	
	if (plyr.diagOn)
	{
		drawText(2,27,"x:   y:   s:");
		drawText(4,27,(plyr.x-1));
		drawText(9,27,(plyr.y-1));
		drawText(14,27,plyr.special);
		drawText(18,27,(plyr.zone));
	}
	
	
	//drawText(17,6,plyr.zoneSet);
	/*
	std::string text;
	std::stringstream out;
	out << plyr.z_offset;
	text = out.str();
	drawText(22,6,text);
	//out << plyr.front;
	//text = out.str();
	//drawText(35,6,text);
	*/
}


void initTextures()
{
	// Load an OpenGL texture.
    // We could directly use a sf::Image as an OpenGL texture (with its Bind() member function),
    // but here we want more control on it (generate mipmaps, ...) so we create a new one
   
    sf::Image Image;
	string filename;
    char tempfilename[100]; 
	glGenTextures(numberOfTextures, &texture[0]); // problem line - don't include in loop. Always 0???
	for ( int i=0; i<numberOfTextures; i++ )
    {	
		filename = textureNames[i];
        sprintf(tempfilename,"%s%s.png","data/images/",filename.c_str());
		Image.LoadFromFile(tempfilename);
		//Image.FlipY(true);

        glBindTexture(GL_TEXTURE_2D, texture[i]);
        gluBuild2DMipmaps(GL_TEXTURE_2D, GL_RGBA, Image.GetWidth(), Image.GetHeight(), GL_RGBA, GL_UNSIGNED_BYTE, Image.GetPixelsPtr());
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	}

	// Need to delete SFML image...

}




void buildLevelView()
{
	//glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	// Draw background
	Background.SetPosition(156.f, 122.f);
	switch (plyr.zoneSet)
    {
    case 0: 
		Background.SetImage(bg14);
		break;
    case 1: 
		Background.SetImage(bg2);
        break;
    case 2: 
		Background.SetImage(bg0);
		break;
	case 3: 
		Background.SetImage(bg1);
		break;
	case 11: 
		Background.SetImage(bg11);
		break;
	case 12: 
		Background.SetImage(bg12);
		break;
	case 13: 
		Background.SetImage(bg13);
		break;
	case 14: 
		Background.SetImage(bg14);
		break;
	case 16: 
		Background.SetImage(bg15);
		break;
	case 18: 
		Background.SetImage(bg16);
		break;
	default:
		Background.SetImage(bg14);
		break;
        }

	if ((plyr.scenario==0) && (plyr.zone==99))
	{
		switch (plyr.facing)
		{
		case 1: 
			Background.SetImage(bgCity4); // west view
			break;
		case 2: 
			Background.SetImage(bgCity3); // north view
			break;
		case 3: 
			Background.SetImage(bgCity1); //east view
			break;
		case 4: 
			Background.SetImage(bgCity2); // south view
			break;
		}
	}
		App.Draw(Background);

	// Start with 5 variables - columns, depth, plyr.x, plyr.y, plyr.facing
	// c and d hold current column and current depth value

	// Draw left hand block of quads
	int leftmostColumn = 0;
	int rightmostColumn = ((columns-1)/2);

	for ( int d=0; d<depth; d++ )
	{
		for ( int c=leftmostColumn; c<rightmostColumn; c++ )
		{
			   calculateWallPositions(c,d);
		}
	}

	// Draw right hand block of quads
	leftmostColumn = ((columns-1)/2)+1;
	rightmostColumn = columns;

	for ( int d=0; d<depth; d++ )
	{
		for ( int c=rightmostColumn; c>(leftmostColumn-1); c-- ) //  int c=leftmostColumn; c<rightmostColumn; c++
		{
			   calculateWallPositions(c,d);
		}
	}
	
	// Draw front block of quads
	int c = ((columns-1)/2); // This should be the central column 13 if columns = 25
	for ( int d=0; d<depth; d++ )
	{
		   calculateWallPositions(c,d);
	}
}



void calculateWallPositions(int c,int d)
{
	// Calculates the actual positions within OpenGL space to draw the 3 quads that make up a map cell
	int x = 0;
	int y = 0;
	switch (plyr.facing)
	{
		case 2: // north
			x = (plyr.x - ((columns-1)/2)) + c; // total colums -1 / 2
			y = ((plyr.y - (depth-1)) + d); // actual depth
			break;

		case 1: // west
			x = (plyr.x - (depth-1)) + d;
			y = (plyr.y + ((columns-1)/2)) - c;
			break;

		case 3: // east
			x = (plyr.x + (depth-1)) - d;
			y = (plyr.y -  ((columns-1)/2)) + c;
			break;

		case 4: // south
			x = (plyr.x + ((columns-1)/2)) - c;
			y = ((plyr.y + (depth-1)) - d);
			break;
	}

	if ( (x > 0) && (x < 65) && (y > 0) && (y < 65) )  // valid location on map? (64 x 64 in example)
	{
		int ind = getMapIndex( x,y );
//		plyr.current_zone = levelmap[ind].zone;
		transMapIndex (ind);
		frontwall = plyr.front; // front wall texture number
		leftwall = plyr.left;   // left wall texture number
		rightwall = plyr.right; // right wall texture number
		specialwall = plyr.specialwall; // special used for guild sign etc in City
		float xm = c*2;           // x float value to be added to texture positioning co-ords
		float zm = d*2;           // z float value to be added to texture positioning co-ords
		zm = (zm+plyr.z_offset)-1.0f; //-1.0f;
		// Draw front, left and right walls for current map cell
		drawCellWalls(c, d, xm, zm, frontwall, leftwall, rightwall); // pass wall numbers and x and z mods
	}
}









int getTextureIndex(int x)
{
	int texture_index;
  
	switch (x)
	{
		case 1:
			texture_index = zones[plyr.zoneSet].arch; // arch image with transparency2 6
			break;
		case 3:
			texture_index = zones[plyr.zoneSet].door; // door1 5
			break;
		case 7:
		case 8:  
		case 9:     // bolted door
		case 10:
		case 11:
		case 12:  
			texture_index = zones[plyr.zoneSet].door; // barred door1 5
			break;
		case 5:
			texture_index = zones[plyr.zoneSet].wall; // secret door1 5
			break;        
		case 13:
			texture_index = zones[plyr.zoneSet].wall; // wall0 4
			break;
		case 27:
			texture_index = 27; // City Shop door
			break;
		case 28:
			texture_index = 28; // City Inn door
			break;
		case 29:
			texture_index = 29; // City Tavern door
			break;
		case 30:
			texture_index = 30; // City Smithy door
			break;
		case 31:
			texture_index = 31; // City Bank door
			break;
		case 32:
			texture_index = 32; // City Guild door
			break;
		case 33:
			texture_index = 33; // City Healer door
			break;
		default:
			texture_index = zones[plyr.zoneSet].wall; // wall0 4
			break;
	}
    return texture_index;
}
