
// Level handling routines

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>

#include <fstream>
#include <iostream>
#include <sstream>
#include <string>

#include "level.h"
#include "player.h"

extern Player plyr;

using std::ofstream;
using namespace std;
using std::cerr;
using std::endl;

//int display_row;
//int levelHeight = 64;
//int levelWidth = 64;

Map maps[5];
string descriptions[255];
string roomMessages[255];
zoneRect zones[100]; // across full first dungeon level
Mapcell levelmap[4096]; // 4096 = 64 by 64 cells

FILE *fp;               // file pointer - used when reading files
char tempString[100];   // temporary string
int i;


Teleport teleports[32] = {

{0xe0,0x12,0x1E,0,0xFF},
{0xe1,0x12,0x1B,0,0x03},
{0xe2,0x13,0x1B,0,0x01},
{0xe3,0x0F,0x17,0,0x00},
{0xe4,0x1D,0x17,0,0x00},
{0xe5,0x05,0x1A,0,0xFF},
{0xe6,0x14,0x1F,0,0x00},
{0xe7,0x10,0x01,0,0xFF},
{0xe8,0x0B,0x01,1,0xFF},
{0xe9,0x11,0x07,1,0xFF},
{0xea,0x11,0x0A,1,0xFF},
{0xeb,0x0B,0x1C,1,0xFF},
{0xec,0x0B,0x1F,1,0xFF},
{0xed,0x08,0x0F,1,0x80},
{0xee,0x12,0x09,2,0x80},
{0xef,0x00,0x01,2,0xFF},
{0xf0,0x00,0x1E,0,0xFF},
{0xf1,0x01,0x00,1,0xFF},
{0xf2,0x1E,0x00,0,0xFF},
{0xf3,0x1F,0x1E,1,0xFF},
{0xf4,0x1F,0x01,3,0xFF},
{0xf5,0x01,0x1F,3,0xFF},
{0xf6,0x1E,0x1F,2,0xFF},
{0xf7,0x00,0x12,5,0xFF},
{0xf8,0x0D,0x0F,5,0xFF},
{0xf9,0x03,0x01,3,0xFF},
{0xfa,0x03,0x1E,1,0xFF},
{0xfb,0x0B,0x00,4,0xFF},
{0xfc,0x16,0x07,1,0x02},
{0xfd,0x1A,0x05,1,0x00},
{0xfe,0x1C,0x07,1,0x02},
{0xff,0x18,0x1B,2,0x03}
};




void moveMapLevel()
{
	if ((plyr.x==51) && (plyr.y==4) && (plyr.map==1)) // to the city
	{
		plyr.x=50;
		plyr.y=4;
		plyr.facing=1;
		plyr.map=0; // The City
		plyr.scenario = 0; // The City
		plyr.z_offset=1.0f;
		loadMapData(0);
		loadDescriptions(0);
		loadZoneData(0);
		loadMessages(0);
	}

	if ((plyr.x==60) && (plyr.y==63) && (plyr.map==1)) // to the city
	{
		plyr.x=60;
		plyr.y=64;
		plyr.facing=4;
		plyr.map=0; // The City
		plyr.scenario = 0; // The City
		plyr.z_offset=1.0f;
		loadMapData(0);
		loadDescriptions(0);
		loadZoneData(0);
		loadMessages(0);
	}

	if ((plyr.x==51)&&(plyr.y==4)&&(plyr.map==0)) // to the dungeon
	{
		plyr.x=50;
		plyr.y=4;
		plyr.facing=1;
		plyr.map=1; // The Dungeon
		plyr.scenario = 1; // The Dungeon
		plyr.z_offset=1.0f;
		loadMapData(1);
		loadDescriptions(1);
		loadZoneData(1);
		loadMessages(1);
	}

	if ((plyr.x==60)&&(plyr.y==63)&&(plyr.map==0)) // to the dungeon
	{
		plyr.x=60;
		plyr.y=62;
		plyr.facing=2;
		plyr.map=1; // The Dungeon
		plyr.scenario = 1; // The Dungeon
		plyr.z_offset=1.0f;
		loadMapData(1);
		loadDescriptions(1);
		loadZoneData(1);
		loadMessages(1);
	}

}


void loadDescriptions(int map)
{
	for (int i=0 ; i<255 ; i++) { descriptions[i]=""; }
	string filename = "data\\map\\"+(maps[map].filename)+"Descriptions.txt";
	std::ifstream instream;
	std::string line;
	instream.open(filename.c_str());
	if ( !instream ) 
	{
      cerr << "Error: " << filename << " file could not be loaded" << endl;
	}
	i = 0;
	while (line!="EOF") 
	{	
		getline(instream, line);
		descriptions[i]= line; 
		i++;
	}
	instream.close();
}


void loadMessages(int map)
{
	for (int i=0 ; i<256 ; i++) { roomMessages[i]=""; }
	string filename = "data\\map\\"+(maps[map].filename)+"Messages.txt";
	std::ifstream instream;
	std::string line;
	instream.open(filename.c_str());
	if ( !instream ) 
	{
      cerr << "Error:" << filename << "could not be loaded" << endl;
	}
	i = 0;
	while (line!="EOF") 
	{	
		getline(instream, line);
		roomMessages[i]= line; 
		i++;
	}
	instream.close();
}


void initMaps()
{
	std::ifstream instream;
	std::string junk,line, text;

	instream.open("data\\map\\maps.txt");
	if( !instream ) 
	{
      cerr << "Error: MAPS.TXT file could not be loaded" << endl;
	}
	i = 0;
	while (junk!="EOF") 
	{	
		//string junk, line, text;
		int fields = 5; // should be 45
		string::size_type idx;
		getline(instream, junk); // read first line as blank
		for (int a = 0; a < fields; ++a) // number of attributes - test for 33
		{
			getline(instream, line);
			//cout << line << "\n";
			idx = line.find('=');
			text = line.substr(idx+2);
			if (a==0) { maps[i].filename = text; }
			if (a==1) { maps[i].width = atoi(text.c_str()); }
			if (a==2) { maps[i].height = atoi(text.c_str()); }
			if (a==3) { maps[i].description = text; }
			if (a==4) { maps[i].background = atoi(text.c_str()); }
		}
		getline(instream, junk);
		//cout << "\n\n";
		i++;
	}
	instream.close();
}



void setCurrentZone()
{
	// x >= entry 3 x2
	// x < entry 1 x1
	// y >= entry 0 y1
	// y < entry 2 y2

	int identifiedZone = 99; // 99 used to represent not part of a zone - default 
	int x = plyr.x-1;
	int y = plyr.y-1;
	for(int z = 0; z < 44; ++z) // 44
	{
		if ( (zones[z].y1<=y) && (zones[z].y2>y) && (zones[z].x1>x) && (zones[z].x2<=x) )
		{
			identifiedZone = z;
		}
	}

	plyr.zone = identifiedZone; 
	plyr.zoneSet = zones[identifiedZone].zoneRef; // set index for image set
	if (identifiedZone == 99) 
	{
		plyr.zoneSet = 0; // set an image set of 0 for default / non zone members
		if ((x>=0) && (x<=31) && (y>=0) && (y<=31)) { plyr.zoneSet = 14; } // map section 1.1	
		if ((x>=32) && (x<=63) && (y>=32) && (y<=63)) { plyr.zoneSet = 14; }  // map section 1.4
		 
		// City override
		if (plyr.map==0) { plyr.zoneSet = 17; }
	
	}

	//temp for city test
	//if (plyr.scenario==0) { plyr.zoneSet = 17; } // city walls
}



void saveHumanReadableMap()
{
	 ofstream outdata; // outdata is like cin
  outdata.open("data\\map\\dungeon1.txt"); // opens the file
   if( !outdata ) 
   { // file couldn't be opened
      cerr << "Error: map file could not be saved" << endl;
      exit(1);
   }
  

int z = 0; // Up to 4096 - current mapcell
for(int y = 0; y < 64; ++y)
{
	for(int x = 0; x < 64; ++x)
	{
		
		//levelmap[z].zone = 0;
		
		outdata << "# Map Cell Item: " << x << "," << y << endl; 
		outdata << "East wall = " << levelmap[z].east << endl;	
		outdata << "North wall = " << levelmap[z].north << endl;
		outdata << "West wall = " << levelmap[z].west << endl;
		outdata << "South wall = " << levelmap[z].south << endl;
		outdata << "Location = " << levelmap[z].location << endl;
		outdata << "Special = " << levelmap[z].special << endl;
		outdata << "      " << endl;
		

		//outdata << x << " " << y << " " << levelmap[z].east << " " << levelmap[z].north << " " << levelmap[z].west << " " << levelmap[z].south << " "
		//	<< levelmap[z].location << " " << levelmap[z].special << " " << endl;
		z++;
	}
}
   outdata.close();
}




void loadZoneData(int map)
{
	
	for (int a = 0; a < 100; ++a)
	{
		zones[a].x1 = 0;
		zones[a].x2 = 0;
		zones[a].y1 = 0;
		zones[a].y2 = 0;
		zones[a].zoneRef = 0;
	}

	string filename = "data\\map\\"+(maps[map].filename)+"Zones.txt";
	std::ifstream instream;
	std::string junk,line, text;
	instream.open(filename.c_str());

	if( !instream ) 
	{
      cerr << "Error:" << filename << " could not be loaded" << endl;
	}
	i = 0;
	while (junk!="EOF") 
	{	
		//string junk, junk2, line, text;
		int fields = 5; 
		string::size_type idx;
		getline(instream, junk); // read first line as blank
		for (int a = 0; a < fields; ++a) // number of attributes
		{
			getline(instream, line);
			idx = line.find('=');
			text = line.substr(idx+2);
			if (a==0) { zones[i].y1 = atoi(text.c_str()); }
			if (a==1) { zones[i].x1 = atoi(text.c_str()); }
			if (a==2) { zones[i].y2 = atoi(text.c_str()); }
			if (a==3) { zones[i].x2 = atoi(text.c_str()); }
			if (a==4) { zones[i].zoneRef = atoi(text.c_str()); }
		}
		getline(instream, junk);
		i++;
	}
	instream.close();
}

void loadMapData(int map)
{
	// # Map Cell Item: 0,0
	// East wall = 0
	// North wall = 13
	// West wall = 13
	// South wall = 0
	// Zone = 0
	// Location = 1
	// Special = 0

	// Need to add in code to clear

	string filename = "data\\map\\"+(maps[map].filename)+"Cells.txt";
	std::ifstream instream;
	std::string junk,junk2, data;
	instream.open(filename.c_str());
   
	if( !instream ) 
	{
		cerr << "Error: terrain file could not be loaded" << endl;
	}

	for(int i = 0; i < 4096; ++i)
	{	
		string junk,line,east,north,west,south,location,special;

		getline(instream, junk);
		getline(instream, line);
		string::size_type idx = line.find('=');
		east = line.substr(idx+2);
		levelmap[i].east = atoi(east.c_str());

		getline(instream, line);
		idx = line.find('=');
		north = line.substr(idx+2);
		levelmap[i].north = atoi(north.c_str());

		getline(instream, line);
		idx = line.find('=');
		west = line.substr(idx+2);
		levelmap[i].west = atoi(west.c_str());

		getline(instream, line);
		idx = line.find('=');
		south = line.substr(idx+2);
		levelmap[i].south = atoi(south.c_str()); 

		getline(instream, line);
		idx = line.find('=');
		location = line.substr(idx+2);
		levelmap[i].location = atoi(location.c_str());
		
		getline(instream, line);
		idx = line.find('=');
		special = line.substr(idx+2);
		levelmap[i].special = atoi(special.c_str());

		getline(instream, junk);

	}

	instream.close();

}



int getMapIndex(int x,int y)
{
    int tmpIndex = ((y-1)*64)+(x-1);
    return tmpIndex;
}



void transMapIndex (int idx)
{
     int facing = plyr.facing;
//	 plyr.current_zone = levelmap[idx].zone;
         
     if (facing == 1) // facing west? 
     {
        plyr.back = levelmap[idx].east;
        plyr.left = levelmap[idx].south;
        plyr.right = levelmap[idx].north;
        plyr.front = levelmap[idx].west;
		
     }
     
     if (facing == 2) // facing north? 
     { 
        plyr.back = levelmap[idx].south;
        plyr.left = levelmap[idx].west;
        plyr.right = levelmap[idx].east;
        plyr.front = levelmap[idx].north;
		
     }
         
     if (facing == 3) // facing east? 
     {
        plyr.back = levelmap[idx].west;
        plyr.right = levelmap[idx].south;
        plyr.left = levelmap[idx].north;
        plyr.front = levelmap[idx].east;
     }
     
     if (facing == 4) // facing south?  
     {
        plyr.back = levelmap[idx].north;
        plyr.right = levelmap[idx].west;
        plyr.left = levelmap[idx].east;
        plyr.front = levelmap[idx].south;
     }

	 plyr.specialwall = levelmap[idx].special;
     
}